@@ -15,17 +15,14 @@ while [ $# -gt 0 ]; do
             echo "  MIPS:    mips, mips64."
             echo "Use --release to build in release mode"
             echo "Use -s to set your keystore file and -p for the password"
-            echo "Use -t to get an AndroidTv build"
             echo "Use -c to get a ChromeOS build"
+            echo "Use -l to build only LibVLC"
             exit 0
             ;;
         a|-a)
             ANDROID_ABI=$2
             shift
             ;;
-        -t)
-            ANDROID_TV=1
-            ;;
         -c)
             CHROME_OS=1
             ;;
@@ -40,6 +37,9 @@ while [ $# -gt 0 ]; do
             PASSWORD_KEYSTORE=$2
             shift
             ;;
+        -l)
+            BUILD_LIBVLC=1
+            ;;
         run)
             RUN=1
             ;;
@@ -131,7 +131,7 @@ fi
 # Fetch VLC source #
 ####################
 
-TESTED_HASH=c4cd717
+TESTED_HASH=6947740
 if [ ! -d "vlc" ]; then
     echo "VLC source not found, cloning"
     git clone git://git.videolan.org/vlc.git vlc
@@ -173,9 +173,7 @@ BUILDTYPE="Debug"
 if [ "$RELEASE" = 1 ]; then
     BUILDTYPE="Release"
 fi
-if [ "$ANDROID_TV" = 1 ]; then
-    PLATFORM="Tv"
-elif [ "$CHROME_OS" = 1 ]; then
+if [ "$CHROME_OS" = 1 ]; then
     PLATFORM="Chrome"
 fi
 
@@ -199,9 +197,14 @@ else
     GRADLE_ABI="ARMv7"
 fi
 
-TARGET="assemble${PLATFORM}${GRADLE_ABI}${BUILDTYPE}"
-
-PASSWORD_KEYSTORE="$PASSWORD_KEYSTORE" ./gradlew $TARGET
+if [ "$BUILD_LIBVLC" = 1 ];then
+    ./gradlew -p libvlc assemble${BUILDTYPE}
+    RUN=0
+    CHROME_OS=0
+else
+    TARGET="assemble${PLATFORM}${GRADLE_ABI}${BUILDTYPE}"
+    PASSWORD_KEYSTORE="$PASSWORD_KEYSTORE" ./gradlew $TARGET
+fi
 
 #######
 # RUN #
