@@ -22,7 +22,7 @@
 
 package org.videolan.libvlc;
 
-@SuppressWarnings("unused")
+@SuppressWarnings("unused, JniMissingFunction")
 public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
 
     public static class Event extends VLCEvent {
@@ -104,6 +104,10 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
     }
 
     public static class Title {
+        private static class Flags {
+            public static final int MENU = 0x01;
+            public static final int INTERACTIVE = 0x02;
+        };
         /**
          * duration in milliseconds
          */
@@ -117,18 +121,26 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
         /**
          * true if the title is a menu
          */
-        public final boolean menu;
+        private final int flags;
 
-        public Title(long duration, String name, boolean menu) {
+        public Title(long duration, String name, int flags) {
             this.duration = duration;
             this.name = name;
-            this.menu = menu;
+            this.flags = flags;
+        }
+
+        public boolean isMenu() {
+            return (this.flags & Flags.MENU) != 0;
+        }
+
+        public boolean isInteractive() {
+            return (this.flags & Flags.INTERACTIVE) != 0;
         }
     }
 
     @SuppressWarnings("unused") /* Used from JNI */
-    private static Title createTitleFromNative(long duration, String name, boolean menu) {
-        return new Title(duration, name, menu);
+    private static Title createTitleFromNative(long duration, String name, int flags) {
+        return new Title(duration, name, flags);
     }
 
     public static class Chapter {
@@ -366,7 +378,7 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
      * @param libVLC a valid libVLC
      */
     public MediaPlayer(LibVLC libVLC) {
-        nativeNewFromLibVlc(libVLC, mWindow);
+        nativeNewFromLibVlc(libVLC, mWindow.getNativeHandler());
     }
 
     /**
@@ -379,7 +391,7 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
             throw new IllegalArgumentException("Media is null or released");
         mMedia = media;
         mMedia.retain();
-        nativeNewFromMedia(mMedia, mWindow);
+        nativeNewFromMedia(mMedia, mWindow.getNativeHandler());
     }
 
     /**
@@ -551,10 +563,15 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
         return nativeSetVideoTrack(index);
     }
 
-    private void setVideoTrackEnabled(boolean enabled) {
+    /**
+     * Set the enabled state of the video track
+     *
+     * @param enabled
+     */
+    public void setVideoTrackEnabled(boolean enabled) {
         if (!enabled) {
             setVideoTrack(-1);
-        } else {
+        } else if (getVideoTrack() == -1) {
             final MediaPlayer.TrackDescription tracks[] = getVideoTracks();
 
             if (tracks != null) {
@@ -828,8 +845,8 @@ public class MediaPlayer extends VLCObject<MediaPlayer.Event> {
     }
 
     /* JNI */
-    private native void nativeNewFromLibVlc(LibVLC libVLC, IAWindowNativeHandler window);
-    private native void nativeNewFromMedia(Media media, IAWindowNativeHandler window);
+    private native void nativeNewFromLibVlc(LibVLC libVLC, AWindowNativeHandler window);
+    private native void nativeNewFromMedia(Media media, AWindowNativeHandler window);
     private native void nativeRelease();
     private native void nativeSetMedia(Media media);
     private native void nativePlay();
