@@ -21,6 +21,7 @@
 package org.videolan.vlc.gui.audio;
 
 import android.Manifest;
+import android.app.Activity;
 import android.content.Context;
 import android.content.Intent;
 import android.content.SharedPreferences;
@@ -30,17 +31,21 @@ import android.os.Handler;
 import android.preference.PreferenceManager;
 import android.support.annotation.RequiresPermission;
 import android.support.design.widget.Snackbar;
+import android.support.design.widget.TextInputLayout;
 import android.support.v4.app.FragmentActivity;
 import android.support.v4.app.FragmentManager;
 import android.support.v7.widget.LinearLayoutManager;
 import android.support.v7.widget.PopupMenu;
 import android.support.v7.widget.RecyclerView;
 import android.support.v7.widget.helper.ItemTouchHelper;
+import android.text.Editable;
+import android.text.TextWatcher;
 import android.view.LayoutInflater;
 import android.view.MenuItem;
 import android.view.MotionEvent;
 import android.view.View;
 import android.view.ViewGroup;
+import android.view.inputmethod.InputMethodManager;
 import android.widget.ImageButton;
 import android.widget.ProgressBar;
 import android.widget.SeekBar;
@@ -50,29 +55,29 @@ import android.widget.ViewSwitcher;
 
 import org.videolan.libvlc.Media;
 import org.videolan.libvlc.MediaPlayer;
-import org.videolan.vlc.gui.helpers.UiTools;
-import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.AudioPlayerContainerActivity;
 import org.videolan.vlc.gui.PlaybackServiceFragment;
+import org.videolan.vlc.gui.dialogs.AdvOptionsDialog;
 import org.videolan.vlc.gui.helpers.SwipeDragItemTouchHelperCallback;
+import org.videolan.vlc.gui.helpers.UiTools;
+import org.videolan.vlc.gui.preferences.PreferencesActivity;
+import org.videolan.vlc.gui.view.AudioMediaSwitcher.AudioMediaSwitcherListener;
 import org.videolan.vlc.gui.view.CoverMediaSwitcher;
 import org.videolan.vlc.gui.view.HeaderMediaSwitcher;
-import org.videolan.vlc.gui.dialogs.AdvOptionsDialog;
-import org.videolan.vlc.gui.dialogs.SavePlaylistDialog;
-import org.videolan.vlc.gui.preferences.PreferencesActivity;
+import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.Strings;
 import org.videolan.vlc.util.Util;
-import org.videolan.vlc.gui.view.AudioMediaSwitcher.AudioMediaSwitcherListener;
 
-import java.util.ArrayList;
 import java.util.List;
 
-public class AudioPlayer extends PlaybackServiceFragment implements PlaybackService.Callback, View.OnClickListener {
+public class AudioPlayer extends PlaybackServiceFragment implements PlaybackService.Callback, View.OnClickListener, PlaylistAdapter.IPlayer, TextWatcher {
     public static final String TAG = "VLC/AudioPlayer";
 
+    public static final int SEARCH_TIMEOUT_MILLIS = 5000;
+
     private ProgressBar mProgressBar;
     private HeaderMediaSwitcher mHeaderMediaSwitcher;
     private CoverMediaSwitcher mCoverMediaSwitcher;
@@ -87,21 +92,23 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
     private ImageButton mShuffle;
     private ImageButton mRepeat;
     private ImageButton mAdvFunc;
-    private ImageButton mPlaylistSwitch, mPlaylistSave;
+    private ImageButton mPlaylistSwitch;
     private SeekBar mTimeline;
-    private RecyclerView mSongsList;
+    private ImageButton mPlaylistSearchButton;
+    private TextInputLayout mPlaylistSearchText;
+    private RecyclerView mPlaylist;
 
     ViewSwitcher mSwitcher;
 
     private boolean mShowRemainingTime = false;
     private boolean mPreviewingSeek = false;
-    private boolean mSwitchedToVideo = false;
 
     private PlaylistAdapter mPlaylistAdapter;
+    private Handler mHandler = new Handler();
 
     private boolean mAdvFuncVisible;
     private boolean mPlaylistSwitchVisible;
-    private boolean mPlaylistSaveVisible;
+    private boolean mSearchVisible;
     private boolean mHeaderPlayPauseVisible;
     private boolean mProgressBarVisible;
     private boolean mHeaderTimeVisible;
@@ -140,18 +147,21 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
         mRepeat = (ImageButton) v.findViewById(R.id.repeat);
         mAdvFunc = (ImageButton) v.findViewById(R.id.adv_function);
         mPlaylistSwitch = (ImageButton) v.findViewById(R.id.playlist_switch);
-        mPlaylistSave = (ImageButton) v.findViewById(R.id.playlist_save);
         mTimeline = (SeekBar) v.findViewById(R.id.timeline);
+        mPlaylistSearchButton = (ImageButton) v.findViewById(R.id.playlist_search);
+        mPlaylistSearchText = (TextInputLayout) v.findViewById(R.id.playlist_search_text);
+        mPlaylistSearchButton.setOnClickListener(this);
+        mPlaylistSearchText.getEditText().addTextChangedListener(this);
 
-        mSongsList = (RecyclerView) v.findViewById(R.id.songs_list);
+        mPlaylist = (RecyclerView) v.findViewById(R.id.songs_list);
         final LinearLayoutManager layoutManager = new LinearLayoutManager(getActivity());
         layoutManager.setOrientation(LinearLayoutManager.VERTICAL);
-        mSongsList.setLayoutManager(layoutManager);
-        mSongsList.setAdapter(mPlaylistAdapter);
+        mPlaylist.setLayoutManager(layoutManager);
+        mPlaylist.setAdapter(mPlaylistAdapter);
 
         ItemTouchHelper.Callback callback =  new SwipeDragItemTouchHelperCallback(mPlaylistAdapter);
         ItemTouchHelper touchHelper = new ItemTouchHelper(callback);
-        touchHelper.attachToRecyclerView(mSongsList);
+        touchHelper.attachToRecyclerView(mPlaylist);
 
         mSwitcher = (ViewSwitcher) v.findViewById(R.id.view_switcher);
         mSwitcher.setInAnimation(getActivity(), android.R.anim.fade_in);
@@ -159,11 +169,11 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
 
         mAdvFuncVisible = false;
         mPlaylistSwitchVisible = false;
-        mPlaylistSaveVisible = false;
+        mSearchVisible = false;
         mHeaderPlayPauseVisible = true;
         mProgressBarVisible = true;
         mHeaderTimeVisible = true;
-        restoreHedaderButtonVisibilities();
+        restoreHeaderButtonVisibilities();
 
         mTime.setOnClickListener(new View.OnClickListener() {
             @Override
@@ -175,8 +185,8 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
             @Override
             public void onClick(View v) {
                 if (mService != null) {
+                    mService.getCurrentMediaWrapper().removeFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
                     mService.switchToVideo();
-                    mSwitchedToVideo = true;
                 }
             }
         });
@@ -230,7 +240,6 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
                 showAdvancedOptions(v);
             }
         });
-        mPlaylistSave.setOnClickListener(this);
         mPlaylistSwitch.setOnClickListener(new View.OnClickListener() {
             @Override
             public void onClick(View v) {
@@ -243,18 +252,13 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
                             R.attr.ic_playlist));
             }
         });
-        registerForContextMenu(mSongsList);
+        registerForContextMenu(mPlaylist);
 
         getActivity().setVolumeControlStream(AudioManager.STREAM_MUSIC);
 
         return v;
     }
 
-    @Override
-    public void onResume() {
-        super.onResume();
-    }
-
     @Override
     public void onPause() {
         super.onPause();
@@ -268,7 +272,7 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
         popupMenu.setOnMenuItemClickListener(new PopupMenu.OnMenuItemClickListener() {
             @Override
             public boolean onMenuItemClick(MenuItem item) {
-                if(item.getItemId() == R.id.audio_player_mini_remove) {
+                if (item.getItemId() == R.id.audio_player_mini_remove) {
                     if (mService != null) {
                         mService.remove(position);
                         return true;
@@ -291,17 +295,15 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
         context.getApplicationContext().sendBroadcast(intent);
     }
 
-    @Override
     public void update() {
         if (mService == null || getActivity() == null)
             return;
 
         if (mService.hasMedia() && !mService.isVideoPlaying()) {
             SharedPreferences mSettings= PreferenceManager.getDefaultSharedPreferences(getActivity());
-            if (mSettings.getBoolean(PreferencesActivity.VIDEO_RESTORE, false)){
+            if (isResumed() && mSettings.getBoolean(PreferencesActivity.VIDEO_RESTORE, false)){
                 Util.commitPreferences(mSettings.edit().putBoolean(PreferencesActivity.VIDEO_RESTORE, false));
                 mService.switchToVideo();
-                mSwitchedToVideo = true;
                 return;
             } else
                 show();
@@ -350,11 +352,32 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
             break;
         }
 
-        final List<String> mediaLocations = mService.getMediaLocations();
-        mShuffle.setVisibility(mediaLocations != null && mediaLocations.size() > 2 ? View.VISIBLE : View.INVISIBLE);
+        mShuffle.setVisibility(mService.getMediaListSize() > 2 ? View.VISIBLE : View.INVISIBLE);
         mTimeline.setOnSeekBarChangeListener(mTimelineListner);
 
-        updateList();
+        if (playlistDiffer())
+            updateList();
+        final int position = mService.getCurrentMediaPosition();
+        if (position != -1) {
+            mPlaylist.post(new Runnable() {
+                @Override
+                public void run() {
+                    mPlaylistAdapter.setCurrentIndex(position);
+                }
+            });
+        }
+    }
+
+    private boolean playlistDiffer() {
+        int serviceListSize = mService.getMediaListSize();
+        if (serviceListSize != mPlaylistAdapter.getItemCount())
+            return true;
+        List<MediaWrapper> adapterList = mPlaylistAdapter.getMedias();
+        List<MediaWrapper> serviceList = mService.getMedias();
+        for (int i = 0 ; i < serviceListSize ; ++i)
+            if (serviceList.get(i) != adapterList.get(i))
+                return true;
+        return false;
     }
 
     @Override
@@ -383,14 +406,7 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
     public void onMediaPlayerEvent(MediaPlayer.Event event) {
         switch (event.type) {
             case MediaPlayer.Event.Opening:
-                mSwitchedToVideo = false;
-                break;
-            case MediaPlayer.Event.ESAdded:
-                final boolean forceAudio = (mService.getCurrentMediaWrapper().getFlags() & MediaWrapper.MEDIA_FORCE_AUDIO) != 0;
-                if (!forceAudio && !mSwitchedToVideo && event.getEsChangedType() == Media.Track.Type.Video) {
-                    mService.switchToVideo();
-                    mSwitchedToVideo = true;
-                }
+                hideSearchField();
                 break;
             case MediaPlayer.Event.Stopped:
                 hide();
@@ -399,23 +415,18 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
     }
 
     public void updateList() {
+        hideSearchField();
         int currentIndex = -1, oldCount = mPlaylistAdapter.getItemCount();
         if (mService == null)
             return;
 
-        final List<MediaWrapper> previousAudioList = mPlaylistAdapter.getMedias();
         mPlaylistAdapter.clear();
 
         final List<MediaWrapper> audioList = mService.getMedias();
-        final String currentItem = mService.getCurrentMediaLocation();
 
         if (audioList != null) {
-            for (int i = 0; i < audioList.size(); i++) {
-                final MediaWrapper media = audioList.get(i);
-                if (currentItem != null && currentItem.equals(media.getLocation()))
-                    currentIndex = i;
-                mPlaylistAdapter.add(media);
-            }
+            mPlaylistAdapter.addAll(audioList);
+            currentIndex = mService.getCurrentMediaPosition();
         }
         mPlaylistAdapter.setCurrentIndex(currentIndex);
         int count = mPlaylistAdapter.getItemCount();
@@ -423,30 +434,20 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
             mPlaylistAdapter.notifyDataSetChanged();
         else
             mPlaylistAdapter.notifyItemRangeChanged(0, count);
+    }
 
-        final int selectionIndex = currentIndex;
-        if (!previousAudioList.equals(audioList))
-            mSongsList.post(new Runnable() {
-                @Override
-                public void run() {
-                    mPlaylistAdapter.setCurrentIndex(selectionIndex);
-                }
-            });
+    @Override
+    public void onSelectionSet(int position) {
+        mPlaylist.smoothScrollToPosition(position);
     }
 
     OnSeekBarChangeListener mTimelineListner = new OnSeekBarChangeListener() {
 
         @Override
-        public void onStopTrackingTouch(SeekBar arg0) {
-            // TODO Auto-generated method stub
-
-        }
+        public void onStopTrackingTouch(SeekBar seekBar) {}
 
         @Override
-        public void onStartTrackingTouch(SeekBar arg0) {
-            // TODO Auto-generated method stub
-
-        }
+        public void onStartTrackingTouch(SeekBar seekBar) {}
 
         @Override
         public void onProgressChanged(SeekBar sb, int prog, boolean fromUser) {
@@ -454,8 +455,8 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
                 mService.setTime(prog);
                 mTime.setText(Strings.millisToString(mShowRemainingTime ? prog- mService.getLength() : prog));
                 mHeaderTime.setText(Strings.millisToString(prog));
+            }
         }
-    }
     };
 
     public void onTimeLabelClick(View view) {
@@ -550,30 +551,30 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
      */
     public void setHeaderVisibilities(boolean advFuncVisible, boolean playlistSwitchVisible,
                                       boolean headerPlayPauseVisible, boolean progressBarVisible,
-                                      boolean headerTimeVisible, boolean playlistSaveVisible) {
+                                      boolean headerTimeVisible, boolean searchVisible) {
         mAdvFuncVisible = advFuncVisible;
         mPlaylistSwitchVisible = playlistSwitchVisible;
         mHeaderPlayPauseVisible = headerPlayPauseVisible;
         mProgressBarVisible = progressBarVisible;
         mHeaderTimeVisible = headerTimeVisible;
-        mPlaylistSaveVisible = playlistSaveVisible;
-        restoreHedaderButtonVisibilities();
+        mSearchVisible = searchVisible;
+        restoreHeaderButtonVisibilities();
     }
 
-    private void restoreHedaderButtonVisibilities() {
-        mAdvFunc.setVisibility(mAdvFuncVisible ? ImageButton.VISIBLE : ImageButton.GONE);
-        mPlaylistSwitch.setVisibility(mPlaylistSwitchVisible ? ImageButton.VISIBLE : ImageButton.GONE);
-        mPlaylistSave.setVisibility(mPlaylistSaveVisible ? ImageButton.VISIBLE : ImageButton.GONE);
-        mHeaderPlayPause.setVisibility(mHeaderPlayPauseVisible ? ImageButton.VISIBLE : ImageButton.GONE);
+    private void restoreHeaderButtonVisibilities() {
+        mAdvFunc.setVisibility(mAdvFuncVisible ? View.VISIBLE : View.GONE);
+        mPlaylistSwitch.setVisibility(mPlaylistSwitchVisible ? View.VISIBLE : View.GONE);
+        mPlaylistSearchButton.setVisibility(mSearchVisible ? View.VISIBLE : View.GONE);
+        mHeaderPlayPause.setVisibility(mHeaderPlayPauseVisible ? View.VISIBLE : View.GONE);
         mProgressBar.setVisibility(mProgressBarVisible ? ProgressBar.VISIBLE : ProgressBar.GONE);
         mHeaderTime.setVisibility(mHeaderTimeVisible ? TextView.VISIBLE : TextView.GONE);
     }
 
-    private void hideHedaderButtons() {
-        mAdvFunc.setVisibility(ImageButton.GONE);
-        mPlaylistSwitch.setVisibility(ImageButton.GONE);
-        mPlaylistSave.setVisibility(ImageButton.GONE);
-        mHeaderPlayPause.setVisibility(ImageButton.GONE);
+    private void hideHeaderButtons() {
+        mAdvFunc.setVisibility(View.GONE);
+        mPlaylistSwitch.setVisibility(View.GONE);
+        mPlaylistSearchButton.setVisibility(View.GONE);
+        mHeaderPlayPause.setVisibility(View.GONE);
         mHeaderTime.setVisibility(TextView.GONE);
     }
 
@@ -594,12 +595,12 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
 
         @Override
         public void onTouchDown() {
-            hideHedaderButtons();
+            hideHeaderButtons();
         }
 
         @Override
         public void onTouchUp() {
-            restoreHedaderButtonVisibilities();
+            restoreHeaderButtonVisibilities();
         }
 
         @Override
@@ -637,19 +638,61 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
     @Override
     public void onClick(View v) {
         switch (v.getId()){
-            case R.id.playlist_save:
-                if (mService == null)
-                    return;
-                FragmentManager fm = getActivity().getSupportFragmentManager();
-                SavePlaylistDialog savePlaylistDialog = new SavePlaylistDialog();
-                Bundle args = new Bundle();
-                args.putParcelableArrayList(SavePlaylistDialog.KEY_TRACKS, (ArrayList<MediaWrapper>) mService.getMedias());
-                savePlaylistDialog.setArguments(args);
-                savePlaylistDialog.show(fm, "fragment_save_playlist");
+            case R.id.playlist_search:
+                mPlaylistSearchButton.setVisibility(View.GONE);
+                mPlaylistSearchText.setVisibility(View.VISIBLE);
+                mPlaylistSearchText.getEditText().requestFocus();
+                InputMethodManager imm = (InputMethodManager) VLCApplication.getAppContext().getSystemService(Context.INPUT_METHOD_SERVICE);
+                imm.showSoftInput(mPlaylistSearchText.getEditText(), InputMethodManager.SHOW_IMPLICIT);
+                mHandler.postDelayed(hideSearchRunnable, SEARCH_TIMEOUT_MILLIS);
                 break;
         }
     }
 
+    @Override
+    public void beforeTextChanged(CharSequence charSequence, int start, int before, int count) {}
+
+    public boolean clearSearch() {
+        mPlaylistAdapter.restoreList();
+        return hideSearchField();
+    }
+
+    public boolean hideSearchField() {
+        if (mPlaylistSearchText.getVisibility() != View.VISIBLE)
+            return false;
+        mPlaylistSearchText.getEditText().removeTextChangedListener(this);
+        mPlaylistSearchText.getEditText().setText("");
+        mPlaylistSearchText.getEditText().addTextChangedListener(this);
+        ((InputMethodManager) VLCApplication.getAppContext().getSystemService(Activity.INPUT_METHOD_SERVICE)).hideSoftInputFromWindow(
+                getActivity().getWindow().getDecorView().getRootView().getWindowToken(), 0);
+        mPlaylistSearchButton.setVisibility(View.VISIBLE);
+        mPlaylistSearchText.setVisibility(View.GONE);
+        return true;
+    }
+
+    Runnable hideSearchRunnable = new Runnable() {
+        @Override
+        public void run() {
+            hideSearchField();
+            mPlaylistAdapter.restoreList();
+        }
+    };
+
+    @Override
+    public void onTextChanged(CharSequence charSequence,  int start, int before, int count) {
+        int length = charSequence.length();
+        if (length > 1) {
+            mPlaylistAdapter.getFilter().filter(charSequence);
+            mHandler.removeCallbacks(hideSearchRunnable);
+        } else if (length == 0) {
+            mPlaylistAdapter.restoreList();
+            hideSearchField();
+        }
+    }
+
+    @Override
+    public void afterTextChanged(Editable editable) {}
+
     @Override
     public void onConnected(PlaybackService service) {
         super.onConnected(service);
@@ -704,11 +747,11 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
                 mTime.setText(Strings.millisToString(mShowRemainingTime ? possibleSeek-length : possibleSeek));
                 mTimeline.setProgress(possibleSeek);
                 mProgressBar.setProgress(possibleSeek);
-                h.postDelayed(seekRunnable, 50);
+                handler.postDelayed(seekRunnable, 50);
             }
         };
 
-        Handler h = new Handler();
+        Handler handler = new Handler();
 
         @Override
         public boolean onTouch(View v, MotionEvent event) {
@@ -724,12 +767,12 @@ public class AudioPlayer extends PlaybackServiceFragment implements PlaybackServ
                 vibrated = false;
                 length = mService.getLength();
 
-                h.postDelayed(seekRunnable, 1000);
+                handler.postDelayed(seekRunnable, 1000);
                 return true;
 
             case MotionEvent.ACTION_UP:
                 (forward ? mNext : mPrevious).setImageResource(this.normal);
-                h.removeCallbacks(seekRunnable);
+                handler.removeCallbacks(seekRunnable);
                 mPreviewingSeek = false;
 
                 if(event.getEventTime()-event.getDownTime() < 1000) {
