@@ -31,10 +31,12 @@ import android.support.v4.app.Fragment;
 import android.text.TextUtils;
 import android.view.View;
 
+import org.videolan.libvlc.util.MediaBrowser;
 import org.videolan.vlc.R;
 import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.FileUtils;
 import org.videolan.vlc.util.Strings;
+import org.videolan.vlc.util.VLCInstance;
 
 public class FilePickerFragment extends FileBrowserFragment {
 
@@ -60,6 +62,7 @@ public class FilePickerFragment extends FileBrowserFragment {
     @Override
     public void onStart() {
         super.onStart();
+        mMediaBrowser = new MediaBrowser(VLCInstance.get(), this);
         mMediaBrowser.setIgnoreFileTypes("db,nfo,ini,jpg,jpeg,ljpg,gif,png,pgm,pgmyuv,pbm,pam,tga,bmp,pnm,xpm,xcf,pcx,tif,tiff,lbm,sfv");
         getActivity().setTitle(getTitle());
     }
