@@ -110,7 +110,7 @@ public class NetworkBrowserFragment extends BaseBrowserFragment implements View.
         switch (id){
             case R.id.network_add_favorite:
                 db = MediaDatabase.getInstance();
-                db.addNetworkFavItem(mw.getUri(), mw.getTitle());
+                db.addNetworkFavItem(mw.getUri(), mw.getTitle(), mw.getArtworkURL());
                 if (isRootDirectory())
                     updateDisplay();
                 return true;
@@ -206,7 +206,7 @@ public class NetworkBrowserFragment extends BaseBrowserFragment implements View.
         if (db.networkFavExists(mCurrentMedia.getUri()))
             db.deleteNetworkFav(mCurrentMedia.getUri());
         else
-            db.addNetworkFavItem(mCurrentMedia.getUri(), mCurrentMedia.getTitle());
+            db.addNetworkFavItem(mCurrentMedia.getUri(), mCurrentMedia.getTitle(), mCurrentMedia.getArtworkURL());
         getActivity().supportInvalidateOptionsMenu();
     }
 
@@ -221,7 +221,10 @@ public class NetworkBrowserFragment extends BaseBrowserFragment implements View.
                     mEmptyView.setVisibility(View.VISIBLE);
                     mRecyclerView.setVisibility(View.GONE);
                 } else {
-                    mEmptyView.setText(mRoot ? R.string.network_shares_discovery : R.string.network_empty);
+                    if (mRoot)
+                        mEmptyView.setText(AndroidDevices.hasLANConnection() ? R.string.network_shares_discovery : R.string.network_connection_needed);
+                    else
+                        mEmptyView.setText(R.string.network_empty);
                     mEmptyView.setVisibility(View.VISIBLE);
                     mRecyclerView.setVisibility(View.GONE);
                     mSwipeRefreshLayout.setRefreshing(false);
