@@ -26,15 +26,26 @@ package org.videolan.vlc.gui.helpers;
 import android.content.Context;
 import android.content.SharedPreferences;
 import android.content.res.TypedArray;
+import android.os.Handler;
+import android.os.Looper;
 import android.preference.PreferenceManager;
 import android.support.annotation.NonNull;
+import android.support.annotation.Nullable;
 import android.support.design.widget.Snackbar;
+import android.text.Html;
 import android.text.TextUtils;
 import android.util.DisplayMetrics;
 import android.util.TypedValue;
 import android.view.View;
+import android.view.animation.Animation;
+import android.view.animation.AnimationSet;
+import android.view.animation.DecelerateInterpolator;
+import android.view.animation.RotateAnimation;
+import android.widget.ImageView;
 import android.widget.TextView;
 
+import org.videolan.vlc.BuildConfig;
+import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 
 import java.util.concurrent.atomic.AtomicInteger;
@@ -43,6 +54,9 @@ public class UiTools {
 
     private static final AtomicInteger sNextGeneratedId = new AtomicInteger(1);
 
+    private static final Handler sHandler = new Handler(Looper.getMainLooper());
+    public static final int DELETE_DURATION = 3000;
+
     /** Print an on-screen message to alert the user */
     public static void snacker(@NonNull View view, @NonNull int stringId) {
         Snackbar.make(view, stringId, Snackbar.LENGTH_SHORT).show();
@@ -53,6 +67,28 @@ public class UiTools {
         Snackbar.make(view, message, Snackbar.LENGTH_SHORT).show();
     }
 
+    /** Print an on-screen message to alert the user, with undo action */
+    public static void snackerWithCancel(@NonNull View view, @NonNull String message, @NonNull final Runnable action) {
+        snackerWithCancel(view, message, action, null);
+    }
+
+    /** Print an on-screen message to alert the user, with undo action */
+    public static void snackerWithCancel(@NonNull View view, @NonNull String message, @NonNull final Runnable action, @Nullable final Runnable cancelAction) {
+        Snackbar.make(view, message, DELETE_DURATION)
+                .setAction(android.R.string.cancel, new View.OnClickListener() {
+                    @Override
+                    public void onClick(View v) {
+                        if (action != null)
+                            sHandler.removeCallbacks(action);
+                        if (cancelAction != null)
+                            cancelAction.run();
+                    }
+                })
+                .show();
+        if (action != null)
+            sHandler.postDelayed(action, DELETE_DURATION);
+    }
+
     public static int convertPxToDp(int px) {
         DisplayMetrics metrics = VLCApplication.getAppResources().getDisplayMetrics();
         float logicalDensity = metrics.density;
@@ -132,4 +168,31 @@ public class UiTools {
         SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
         return pref.getBoolean("enable_black_theme", false);
     }
+
+    public static void fillAboutView(View v) {
+        TextView link = (TextView) v.findViewById(R.id.main_link);
+        link.setText(Html.fromHtml(VLCApplication.getAppResources().getString(R.string.about_link)));
+
+        String revision = VLCApplication.getAppResources().getString(R.string.build_revision);
+        String builddate = VLCApplication.getAppResources().getString(R.string.build_time);
+        String builder = VLCApplication.getAppResources().getString(R.string.build_host);
+
+        TextView compiled = (TextView) v.findViewById(R.id.main_compiled);
+        compiled.setText(builder + " (" + builddate + ")");
+        TextView textview_rev = (TextView) v.findViewById(R.id.main_revision);
+        textview_rev.setText(VLCApplication.getAppResources().getString(R.string.revision) + " " + revision + " (" + builddate + ") " + BuildConfig.FLAVOR_abi);
+
+        final ImageView logo = (ImageView) v.findViewById(R.id.logo);
+        logo.setOnClickListener(new View.OnClickListener() {
+            @Override
+            public void onClick(View v) {
+                AnimationSet anim = new AnimationSet(true);
+                RotateAnimation rotate = new RotateAnimation(0f, 360f, Animation.RELATIVE_TO_SELF, 0.5f, Animation.RELATIVE_TO_SELF, 0.5f);
+                rotate.setDuration(800);
+                rotate.setInterpolator(new DecelerateInterpolator());
+                anim.addAnimation(rotate);
+                logo.startAnimation(anim);
+            }
+        });
+    }
 }
